import { Injectable, OnModuleInit } from '@nestjs/common';
import { DiscoveryService, Reflector as CoreReflector } from '@nestjs/core';
import { ALLOW_PERMISSIONS_KEY } from '../guards/permission.guard';

@Injectable()
export class PermissionsScannerService implements OnModuleInit {
    private readonly _permissions: Set<string> = new Set<string>();

    //#region Constructor
    constructor(private readonly _discoveryService: DiscoveryService, private readonly _reflector: CoreReflector) {}
    //#endregion

    //#region Getters
    public get permissions(): string[] {
        return Array.from(this._permissions);
    }
    //#endregion

    //#region Public APIs
    async onModuleInit() {
        const controllers = this._discoveryService.getControllers();

        // Scan controller methods
        for (const wrapper of controllers) {
            const { instance } = wrapper;
            if (!instance) continue;

            const prototype = Object.getPrototypeOf(instance);
            const methodNames = Object.getOwnPropertyNames(prototype);

            for (const methodName of methodNames) {
                const method = prototype[methodName];

                if (typeof method !== 'function') continue;
                const args: { keywords: string[] } = this._reflector.get(ALLOW_PERMISSIONS_KEY, method);

                if (args && args.keywords) {
                    args.keywords.forEach((k) => this._permissions.add(k));
                }
            }
        }
    }
    //#endregion
}
